; ZLib v1.2

	.org	$0000

#include	"joeti83.inc"

#define	data	cmdshad+38	; 2 bytes of data

	.dw	beep	; beep 0
	.dw	sprite	; draw a sprite 1
	.dw	rand	; random number 2
	.dw	getbit	; get the bit for a pixel 3
	.dw	detect	; detect a program in the vat 4
	.dw	sleep	; shut down the calc 5
	.dw	decomp	; decompress level data 6
	.dw	delete	; delete a program and restart SOS 7
	.dw	contup	; turn up the contrast 8
	.dw	contdn	; turn down the contrast 9
	.dw	contst	; set the contrast A
	.dw	invert	; invert the screen B
	.dw	getpix	; find a pixel C
	.dw	hiscr	; check for/set high score D
	.dw	error
	.dw	error

;---------= Make a sound =---------
; c=frequency, b=duration
beep:	di
	ld		e,$D0
beepl1:	ld		a,c
beepl2:	dec		a
	jr		nz,beepl2
	ld		a,e
	xor		%00000011
	ld		e,a
	out		(0),a
	djnz		beepl1
	ei
error:	ret

;---------= XOR a sprite =---------
; b=size of sprite
; l=yc
; a=xc
; ix holds pointer
sprite:	ld	e,l
	ld	h,$00
	ld	d,h
	add	hl,de
	add	hl,de
	add	hl,hl
	add	hl,hl
	ld	e,a
	and	$07
	ld	c,a
	srl	e
	srl	e
	srl	e
	add	hl,de
	ld	de,gbuf
	add	hl,de
	push	ix
	pop	de
sl1:	push	bc
	ld	a,(de)
	ld	b,a
	ld	a,c
	or	a
	jr	z,sl3
	ld	c,$00
sl2:	srl	b
	rr	c
	dec	a
	jr	nz,sl2
sl3:	ld	a,(hl)
	xor	b
	ld	(hl),a
	inc	hl
	ld	a,(hl)
	xor	c
	ld	(hl),a
	ld	bc,$0B
	add	hl,bc
	inc	de
	pop	bc
	djnz	sl1
	ret

;---------= Random number generator =---------
; input b=upper bound
; ouput a=answer 0<=a<b
; all registers are preserved except: af and bc
rand:	push	hl
	push	de
	ld	a,(data)
	ld	c,a
	xor	a
	add	a,c
	sla	c
	sla	c
	add	a,c
	sla	c
	sla	c
	add	a,c
	inc	a
	ld	(data),a
	rra
	ld	hl,$0000
	ld	e,a
	ld	d,h
randl:	add	hl,de
	djnz	randl
	ld	a,h
	pop	de
	pop	hl
nomore:	ret

;---------= Get location of a pixel =---------
; input:	e=y coordinate
;		a=x coordinate
; output:	a holds data for pixel (e.g. %00100000)
;		hl->byte where pixel is on the gbuf
getpix:	ld	d,$00
	ld	hl,gbuf
	add	hl,de
	add	hl,de
	add	hl,hl
	add	hl,hl

;---------= Get the bit for a pixel =---------
; input:	a - x coordinate
;		hl - start location	; includes gbuf
; returns:	a - holds bit
;		hl - location + x coordinate/8
;		b=0
;		c=a/8
getbit:	ld		b,$00
	ld		c,a
	and		%00000111
	srl		c
	srl		c
	srl		c
	add		hl,bc
	ld		b,a
	inc		b
	ld		a,%00000001
gblp:	rrca
	djnz		gblp
	ret

;---------= Detect Program =---------
; input:
;	hl=place to start looking
;	ix->first line of data (0 terminated)
; output:
;	de=place stopped + 1
;	hl->program data (after the string)
;	z=0 if found, z=1 if not found.
;	most registers are messed up
detect:
chgwlp:	ld	(data),hl	; in case a program is found
	ld	a,(hl)		; 	but no good.
	and	%00011111
	cp	$06
	jr	z,cwchck
	cp	$05
	jr	z,nogood2	; a program, but not protected
	cp	$01		; (using an unprotected program
	jr	z,list		; could be dangerous)
	cp	$0D
	jr	z,list
	ret
cwchck:	dec	hl		; move to ptr (skip var type)
	push	hl
	ld	b,(hl)
	dec	hl
	ld	c,(hl)
	ld	h,c
	ld	l,b	; now we are at the program
	inc	hl	; skip file length
	inc	hl
	push	ix
	pop	de
ckprog:	ld	c,(hl)
	ld	a,(de)
	cp	c
	jr	nz,nogood
	inc	de
	inc	hl
	ld	a,(de)
	or	a
	jr	nz,ckprog
	ex	de,hl
	pop	hl	; back at the fat
	dec	hl
	dec	hl
	ld	b,(hl)		; length of name
	inc	b		; move to next program as well in case the
skprog:	dec	hl		; user doen't want this world
	djnz	skprog
	ex	de,hl
	xor	a
	ret
nogood2:
	push	hl
nogood:	pop	hl
	ld	hl,(data)
list:	dec	hl		; lists enter here to be skipped
	dec	hl
	dec	hl
	ld	b,(hl)
nogdlp:	dec	hl
	djnz	nogdlp
	dec	hl		; next item
	jr	chgwlp

;--------= Sleep =---------
; shut down the calculator until user presses on.
sleep:	di
	ld	a,$01
	out	(3),a
	xor	a
	out	(4),a
	ei
	res	4,(iy+9)
	jp	getkey

;---------= Decompress =---------
; input:
;	hl->compressed data
;	de->place to load data
;	b=length of compressed data
;	c=compression factor (1, 3, or 15)
; output:
;	level is decompressed
;	hl->next byte of compressed data
decomp:	push	bc
	ld	a,(hl)
	ld	(data),a
	ld	a,c
	ld	b,$08
	cp	$0F
	jr	nz,dmpc
	ld	b,$02
dmpc:	cp	$03
	jr	nz,dcmp1
	ld	b,$04
dcmp1:	push	bc
	ld	a,c
	ld	b,$01
	cp	$0F
	jr	nz,dcmpa
	ld	b,$04
dcmpa:	cp	$03
	jr	nz,dcmpb
	ld	b,$02
dcmpb:	ld	a,(data)
dcmp2:	rlca
	djnz	dcmp2
	ld	(data),a
	and	c
	ld	(de),a
	inc	de
	pop	bc
	djnz	dcmp1
	inc	hl
	pop	bc
	djnz	decomp
	ret

;---------= Delete =---------
; input:
;	hl->name of program to delete (0 terminated)
; output:
;	deletes program and exits back to SOS
delete:	ld	a,$06
	ld	(op1),a
	ld	de,op1+1
	ld	bc,$0008
	ldir
	pop	hl
	pop	hl
	pop	de
	pop	bc
dellp:	dec	hl
	dec	de
	call	$9171
	jr	nz,dellp
	call	chksysvar
	call	nc,delvar
	xor	a
	ld	($9635),a
	ld	($9637),a
	ld	($9638),a
	inc	a
	ld	($9636),a
	jp	$9327

;---------= Change the contrast =--------
; three routines.
; input for contup/contdn (contrast up/contrast down): nothing
; input for contst (contrast set): a=constrast (0-63)
; output: 
contup:	ld	a,($8008)
	cp	63
	ret	z
	inc	a
	jr	contst
contdn:	ld	a,($8008)
	or	a
	ret	z
	dec	a
contst:	ld	($8008),a
	add	a,$18
	or	$C0
	out	($10),a
	ret

;---------= Invert the screen =---------
; Input: nothing
; Output: screen is inverted (text included)
invert:	ld	hl,gbuf
	ld	bc,768
invlp:	ld	a,(hl)
	cpl
	ld	(hl),a
	inc	hl
	dec	bc
	ld	a,b
	or	c
	jr	nz,invlp
	bit	3,(iy+5)
	set	3,(iy+5)
	ret	z
	res	3,(iy+5)
	ret

;---------= High Score =---------
; Input: de=previous high score
;	hl=current score
; Output: hl=high score
;	z=1 (a=0) if new high score, z=0 (a=1) if not
; Registers destroyed: af, de, hl
hiscr:	push	hl
	xor	a
	sbc	hl,de
	jr	c,nohs
	jr	z,nohs
	pop	hl
	or	a
	ret
nohs:	pop	hl
	ex	de,hl
	inc	a
	ret

.end
END